<?php namespace App\Controllers;

use App\Models\UserModel;
use CodeIgniter\Controller;

class Auth extends Controller
{
    protected $userModel;
    
    public function __construct()
    {
        $this->userModel = new UserModel();
    }
    
    // Display registration form
    public function register()
    {
        echo view('auth/register');  // This loads app/Views/auth/register.php
    }
    
// Process the registration form submission
public function processRegister()
{
    helper(['form', 'url']);

    $rules = [
        'name'     => 'required|min_length[3]',
        'email'    => 'required|valid_email|is_unique[users.email]',
        'password' => 'required|min_length[6]',
    ];

    if (!$this->validate($rules)) {
        return redirect()->back()->withInput()->with('errors', $this->validator->getErrors());
    }

    $token = bin2hex(random_bytes(16));  // Generate a token for email verification

    $data = [
        'name'                => $this->request->getPost('name'),
        'email'               => $this->request->getPost('email'),
        'password'            => password_hash($this->request->getPost('password'), PASSWORD_DEFAULT),
        'role_id'             => 4, // Default role as Participant
        'email_verified'      => 0, // Mark as unverified
        'verification_token'  => $token,
    ];

    // Insert user into database
    $db = \Config\Database::connect();
    $builder = $db->table('users');
    $builder->insert($data);

    // Send verification email
    $this->_sendVerificationEmail($data['email'], $token);

    return redirect()->to('/register-success')->with('message', 'Registration successful! Please check your email for verification.');
}

    
// Private method to send verification email
private function _sendVerificationEmail($email, $token)
{
    $emailService = \Config\Services::email();
    $emailService->setTo($email);
    $emailService->setFrom('info@winslowsolutions.com', 'Winslow Solutions');
    $emailService->setSubject('Verify Your Email - Winslow Solutions');

    // Generate verification link
    $verifyLink = site_url("auth/verifyEmail/$token");

    // Professional HTML Email Template
    $message = '
    <html>
    <head>
        <meta charset="UTF-8">
        <title>Email Verification</title>
        <style>
            body { font-family: Arial, sans-serif; background-color: #f4f4f4; padding: 20px; }
            .container { max-width: 600px; margin: auto; background: #ffffff; padding: 20px; border-radius: 8px; box-shadow: 0 0 10px rgba(0, 0, 0, 0.1); }
            .header { background: #164986; padding: 15px; text-align: center; color: #ffffff; font-size: 20px; font-weight: bold; }
            .content { padding: 20px; text-align: center; color: #333; font-size: 16px; }
            .verify-button { display: inline-block; background: #1D4ED8; color: #ffffff; padding: 12px 20px; text-decoration: none; font-size: 16px; border-radius: 5px; margin-top: 20px; }
            .footer { margin-top: 20px; font-size: 14px; color: #777; text-align: center; }
        </style>
    </head>
    <body>
        <div class="container">
            <div class="header">Winslow Solutions</div>
            <div class="content">
                <p>Hello,</p>
                <p>Thank you for registering with Winslow Solutions! Please verify your email by clicking the button below:</p>
                <p><a href="' . $verifyLink . '" class="verify-button">Verify Email</a></p>
                <p>If you did not sign up, you can safely ignore this email.</p>
            </div>
            <div class="footer">
                &copy; ' . date("Y") . ' Winslow Solutions. All rights reserved.
            </div>
        </div>
    </body>
    </html>';

    // Set the email to HTML format
    $emailService->setMailType('html');
    $emailService->setMessage($message);
    $emailService->send();
}
    
    // Verify email when user clicks the verification link
public function verifyEmail($token = null)
{
    if (!$token) {
        return redirect()->to('/login')->with('error', 'Invalid verification token.');
    }

    // Find user by token
    $user = $this->userModel->where('verification_token', $token)->first();

    if ($user) {
        // Update user to verified
        $this->userModel->update($user['id'], [
            'email_verified' => 1,
            'verification_token' => null
        ]);
        return redirect()->to('/login')->with('message', 'Email verified successfully! You can now log in.');
    }

    return redirect()->to('/login')->with('error', 'Invalid or expired verification token.');
}

    
    // Display login form
    public function login()
    {
        echo view('auth/login');  // This loads app/Views/auth/login.php
    }
    
    // Process login form submission
    public function processLogin()
    {
        helper(['form', 'url']);
        $rules = [
            'email'    => 'required|valid_email',
            'password' => 'required'
        ];
        
        if (!$this->validate($rules)) {
            return redirect()->back()->withInput()->with('errors', $this->validator->getErrors());
        }
        
        $email    = $this->request->getPost('email');
        $password = $this->request->getPost('password');
        $remember = $this->request->getPost('remember');
        
        $user = $this->userModel->where('email', $email)->first();
        
        if (!$user) {
            return redirect()->back()->with('error', 'Invalid credentials.');
        }
        
        // Check if email is verified
        if (!$user['email_verified']) {
            return redirect()->back()->with('error', 'Please verify your email before logging in.');
        }
        
        if (password_verify($password, $user['password'])) {
            // Set session data
            $sessionData = [
                'user_id'   => $user['id'],
                'email'     => $user['email'],
                'role_id'   => $user['role_id'],
                'logged_in' => true,
            ];
            session()->set($sessionData);
            
            // Set session expiration to 3 hours (10800 seconds)
            session()->setTempdata('user_session', true, 10800);
            
            // Handle "Remember Me" (example using a simple cookie)
            if ($remember) {
                set_cookie('remember_me', $user['id'], 10800);
            }
            
            return redirect()->to('/dashboard')->with('message', 'Login successful.');
        }
        return redirect()->back()->with('error', 'Invalid credentials.');
    }
    
    // Display forgot password form
    public function forgotPassword()
    {
        return view('auth/forgot_password'); // This loads app/Views/auth/forgot_password.php
    }
    
    // Process forgot password form submission
public function processForgotPassword()
{
    helper(['form', 'url']);
    $rules = [
        'email' => 'required|valid_email'
    ];

    if (!$this->validate($rules)) {
        return redirect()->back()->withInput()->with('errors', $this->validator->getErrors());
    }

    $email = $this->request->getPost('email');
    $user  = $this->userModel->where('email', $email)->first();

    if (!$user) {
        return redirect()->back()->with('error', 'If an account with this email exists, a password reset email will be sent.');
    }

    // Generate reset token and update the user record
    $resetToken = bin2hex(random_bytes(16));
    $this->userModel->update($user['id'], ['password_reset_token' => $resetToken]);

    // Send password reset email
    $this->_sendPasswordResetEmail($email, $resetToken);

    // Redirect to login with a confirmation message
    return redirect()->to('/login')->with('message', 'If an account with this email exists, a password reset link has been sent.');
}

    
    // Private method to send a password reset email
// Private method to send a password reset email
private function _sendPasswordResetEmail($email, $token)
{
    $emailService = \Config\Services::email();
    $emailService->setTo($email);
    $emailService->setFrom('info@winslowsolutions.com', 'Winslow Solutions');
    $emailService->setSubject('Reset Your Password - Winslow Solutions');

    // Generate the password reset link
    $resetLink = site_url("auth/resetPassword/$token");

    // Professional HTML Email Template
    $message = '
    <html>
    <head>
        <meta charset="UTF-8">
        <title>Password Reset Request</title>
        <style>
            body {
                font-family: Arial, sans-serif;
                background-color: #f4f4f4;
                margin: 0;
                padding: 0;
            }
            .container {
                max-width: 600px;
                margin: 20px auto;
                background: #ffffff;
                padding: 20px;
                border-radius: 8px;
                box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);
            }
            .header {
                background: #164986;
                padding: 15px;
                text-align: center;
                color: #ffffff;
                font-size: 20px;
                font-weight: bold;
                border-top-left-radius: 8px;
                border-top-right-radius: 8px;
            }
            .content {
                padding: 20px;
                text-align: center;
                color: #333;
                font-size: 16px;
                line-height: 1.6;
            }
            .reset-button {
                display: inline-block;
                background: #1D4ED8;
                color: #ffffff;
                padding: 12px 20px;
                text-decoration: none;
                font-size: 16px;
                font-weight: bold;
                border-radius: 5px;
                margin-top: 20px;
            }
            .footer {
                margin-top: 20px;
                font-size: 14px;
                color: #777;
                text-align: center;
            }
        </style>
    </head>
    <body>
        <div class="container">
            <div class="header">Winslow Solutions</div>
            <div class="content">
                <p>Hi,</p>
                <p>You recently requested to reset your password for your Winslow Solutions account.</p>
                <p>Click the button below to reset your password:</p>
                <p>
                    <a href="' . $resetLink . '" class="reset-button">Reset Password</a>
                </p>
                <p>If you did not request a password reset, please ignore this email.</p>
                <p>If you need any assistance, feel free to contact our support team.</p>
            </div>
            <div class="footer">
                &copy; ' . date("Y") . ' Winslow Solutions. All rights reserved.
            </div>
        </div>
    </body>
    </html>';

    // Set the email to HTML format
    $emailService->setMailType('html');
    $emailService->setMessage($message);

    // Send the email
    $emailService->send();
}

    
    // Display reset password form
    public function resetPassword($token = null)
    {
        if (!$token) {
            return redirect()->to('/login')->with('error', 'Invalid token.');
        }
        
        $data['token'] = $token;
        echo view('auth/reset_password', $data);  // This loads app/Views/auth/reset_password.php
    }
    
    // Process reset password form submission
public function processResetPassword($token = null)
{
    helper(['form', 'url']);
    if (!$token) {
        return redirect()->to('/login')->with('error', 'Invalid or expired token.');
    }

    $rules = [
        'password'         => 'required|min_length[6]',
        'password_confirm' => 'matches[password]'
    ];

    if (!$this->validate($rules)) {
        return redirect()->back()->withInput()->with('errors', $this->validator->getErrors());
    }

    $user = $this->userModel->where('password_reset_token', $token)->first();
    if (!$user) {
        return redirect()->to('/login')->with('error', 'Invalid or expired token.');
    }

    // Update the password and clear the reset token
    $this->userModel->update($user['id'], [
        'password'             => password_hash($this->request->getPost('password'), PASSWORD_BCRYPT),
        'password_reset_token' => null
    ]);

    // Redirect with a success message
    return redirect()->to('/login')->with('message', 'Your password has been successfully reset. You can now log in.');
}

    
    // Display registration success page
    public function registerSuccess()
    {
        return view('auth/register_success'); // This loads app/Views/auth/register_success.php
    }
    
//  Edit User
    
    public function editUser($id)
{
    // Ensure only Admins can access this
    if (session()->get('role_id') != 1) {
        return redirect()->to('/dashboard')->with('error', 'Unauthorized access.');
    }

    // Fetch user details
    $user = $this->userModel->find($id);
    if (!$user) {
        return redirect()->to('/dashboard')->with('error', 'User not found.');
    }

    return view('auth/edit_user', ['user' => $user]);
}

// Process User Update (Admin)
public function updateUser($id)
{
    if (session()->get('role_id') != 1) {
        return redirect()->to('/dashboard')->with('error', 'Unauthorized access.');
    }

    helper(['form', 'url']);
    $rules = [
        'name'  => 'required|min_length[3]',
        'email' => 'required|valid_email',
        'role_id' => 'required|integer'
    ];

    if (!$this->validate($rules)) {
        return redirect()->back()->withInput()->with('errors', $this->validator->getErrors());
    }

    $data = [
        'name'  => $this->request->getPost('name'),
        'email' => $this->request->getPost('email'),
        'role_id' => $this->request->getPost('role_id'),
    ];

    $this->userModel->update($id, $data);
    return redirect()->to('/dashboard')->with('message', 'User updated successfully.');
}

// Edit Profile
public function editProfile()
{
    // Ensure user is logged in
    if (!session()->get('logged_in')) {
        return redirect()->to('/login')->with('error', 'You must be logged in to edit your profile.');
    }

    // Fetch user details
    $id = session()->get('user_id');
    $user = $this->userModel->find($id);

    return view('auth/edit_profile', ['user' => $user]);
}

// Process User Profile Update
public function updateProfile()
{
    if (!session()->get('logged_in')) {
        return redirect()->to('/login')->with('error', 'You must be logged in to update your profile.');
    }

    helper(['form', 'url']);
    $rules = [
        'name'  => 'required|min_length[3]',
        'email' => 'required|valid_email',
    ];

    if (!$this->validate($rules)) {
        return redirect()->back()->withInput()->with('errors', $this->validator->getErrors());
    }

    $id = session()->get('user_id');
    $data = [
        'name'  => $this->request->getPost('name'),
        'email' => $this->request->getPost('email'),
    ];

    $this->userModel->update($id, $data);
    return redirect()->to('/dashboard')->with('message', 'Profile updated successfully.');
}

// Log Out
public function logout()
{
    session()->destroy(); // Destroy session
    return redirect()->to('/login')->with('message', 'You have been logged out.');
}

    
    
}



