<?php
// ===========================================
// 🟢 Initialization & Configuration
// ===========================================
error_reporting(E_ALL);
ini_set('display_errors', 1);

// Force no caching for the browser.
session_cache_limiter('nocache');
header("Cache-Control: no-cache, no-store, must-revalidate");
header("Pragma: no-cache");
header("Expires: 0");

// ===========================================
// 🟢 Helper Functions
// ===========================================

// Base URL function to dynamically create paths
if (!function_exists('base_url')) {
    function base_url($path = '') {
        $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') ? "https://" : "http://";
        $host = $_SERVER['HTTP_HOST'];
        $scriptDir = rtrim(dirname($_SERVER['SCRIPT_NAME']), '/\\');
        return $protocol . $host . $scriptDir . '/' . ltrim($path, '/');
    }
}

// Escape function to sanitize output
if (!function_exists('esc')) {
    function esc($string) {
        return htmlspecialchars($string, ENT_QUOTES, 'UTF-8');
    }
}

// ===========================================
// 🟢 Autoload Modules
// ===========================================
require_once __DIR__ . '/autoload.php';

// Load all module PHP files dynamically
$modulesDir = __DIR__ . '/modules/';
$moduleFiles = glob($modulesDir . '*.php');
$moduleMarkup = [];
$modules = []; // Define this outside the loop to avoid redefinitions

foreach ($moduleFiles as $file) {
    require_once $file;
    $moduleName = basename($file, '.php');
    if (class_exists($moduleName)) {
        $moduleInstance = new $moduleName();
        $moduleMarkup[$moduleName] = $moduleInstance->render();
        $modules[] = [
            'name' => $moduleName,
            'buttonName' => $moduleInstance->settings['buttonName'] ?? $moduleName,
            'position' => $moduleInstance->settings['buttonPosition'] ?? 999,
        ];
    }
}

// Sort modules by position for better UI arrangement
usort($modules, fn($a, $b) => $a['position'] <=> $b['position']);
?>

<!DOCTYPE html>
<html lang="en">
<head>
    <!-- ===========================================
         🟢 Page Metadata & Stylesheets
    =========================================== -->
    <meta charset="UTF-8">
    <title>App Creator Dashboard</title>
    
    <!-- Prevent browser caching -->
    <meta http-equiv="Cache-Control" content="no-cache, no-store, must-revalidate">
    <meta http-equiv="Pragma" content="no-cache">
    <meta http-equiv="Expires" content="0">
    
    <!-- Google Fonts -->
    <link href="https://fonts.googleapis.com/css?family=Roboto:400,500,700&display=swap" rel="stylesheet">

    <!-- Main Application Styles -->
    <link rel="stylesheet" href="<?php echo esc(base_url('assets/css/app_styles.css')); ?>?v=<?php echo time(); ?>">
</head>
<body>

    <!-- ===========================================
         🟢 Top Navigation Bar (App Name, Controls)
    =========================================== -->
    <div class="top-bar">
        <h1>App Creator Dashboard</h1>
        <div class="app-info">
            <input type="text" id="app-name" placeholder="App Name">
            <input type="text" id="app-slug" placeholder="App Slug">
            <button id="saveApp" class="btn save-btn">Save Application</button>
        </div>
        <div class="controls">
            <button class="btn undo-btn">Undo</button>
            <button class="btn redo-btn">Redo</button>
        </div>
    </div>

    <!-- ===========================================
         🟢 Main Layout Sections
    =========================================== -->
    <div id="app-builder">
        
        <!-- Sidebar: List of Available Modules -->
        <div class="sidebar">
            <h2>Modules</h2>
            <ul>
                <!-- Render module selection buttons -->
                <?php foreach ($modules as $mod): ?>
                    <li>
                        <button class="module-selector-btn" data-module="<?php echo esc($mod['name']); ?>" draggable="true">
                            <?php echo esc($mod['buttonName']); ?>
                        </button>
                    </li>
                <?php endforeach; ?>
            </ul>
        </div>

        <!-- Main Canvas: Where Modules are Placed -->
        <div class="canvas" id="canvas">
            <h2>Your App Preview</h2>
        </div>

        <!-- Module Settings Panel -->
        <div class="settings" id="settings-pane">
            <div id="settings-content">
                <p>Select a module to edit its settings.</p>
            </div>
        </div>

    </div>

    <!-- ===========================================
         🟢 JavaScript Section
    =========================================== -->
    
    <script src="https://main.winslowapps.com/js/tinymce/tinymce.min.js"></script>

    
    
    <script>
        // Holds the rendered HTML for each module
        const moduleMarkup = <?php echo json_encode($moduleMarkup); ?>;

        // Holds the settings UI for each module (or null if no settings exist)
        const moduleSettingsMarkup = {
            <?php
            foreach ($moduleFiles as $file) {
                require_once $file;
                $moduleName = basename($file, '.php');
                if (class_exists($moduleName)) {
                    $instance = new $moduleName();
                    if (method_exists($instance, 'renderSettings')) {
                        echo '"' . esc($moduleName) . '": ' . json_encode($instance->renderSettings()) . ',';
                    } else {
                        echo '"' . esc($moduleName) . '": null,';
                    }
                }
            }
            ?>
        };
    </script>

    <!-- ===========================================
         🟢 JavaScript File Inclusions
    =========================================== -->
    <!-- Settings Panel Management -->
    <script src="<?php echo esc(base_url('assets/js/settings-panel.js')); ?>?v=<?php echo time(); ?>"></script>
    
    <!-- Module Drag & Drop Builder -->
    <script src="<?php echo esc(base_url('assets/js/module-builder.js')); ?>?v=<?php echo time(); ?>"></script>

    <!-- Application Saving & Storage -->
    <script src="<?php echo esc(base_url('assets/js/app-saver.js')); ?>?v=<?php echo time(); ?>"></script>

</body>
</html>
