<?php
// modules/PageTextModule.php

ini_set('display_errors', 1);
error_reporting(E_ALL);

// If not already defined
if (!function_exists('esc')) {
    function esc($string) {
        return htmlspecialchars($string, ENT_QUOTES, 'UTF-8');
    }
}

class PageTextModule {
    private $settings;
    private $instanceId;

    public function __construct($instanceId = '', $settings = []) {
        $this->instanceId = $instanceId ?: uniqid('pageText_');
        // Default text & styles
        $this->settings = array_merge([
            'content'       => 'Many desktop publishing packages and web page editors now use Lorem Ipsum as their default model text.',
            'fontSize'      => '20px',
            'fontWeight'    => 'bold',
            'textAlign'     => 'left',
            'textBoxAlign'  => 'flex-start',
            'width'         => '60%',
            'paddingTop'    => '10px',
            'paddingRight'  => '10px',
            'paddingBottom' => '10px',
            'paddingLeft'   => '10px',
            'marginTop'     => '10px',
            'marginRight'   => 'auto',
            'marginBottom'  => '10px',
            'marginLeft'    => 'auto',
        ], $settings);
    }

    /**
     * Render the module in the canvas.
     */
    public function render() {
        ob_start(); ?>
        
        <div class="module-preview page-text-module"
             data-module="PageTextModule"
             data-module-id="<?php echo esc($this->instanceId); ?>"
             style="display: flex; justify-content: <?php echo esc($this->settings['textBoxAlign']); ?>;"
        >
            <div class="text-container"
                 style="width: <?php echo esc($this->settings['width']); ?>;
                        text-align: <?php echo esc($this->settings['textAlign']); ?>;
                        font-size: <?php echo esc($this->settings['fontSize']); ?>;
                        font-weight: <?php echo esc($this->settings['fontWeight']); ?>;
                        font-family: 'Open Sans', sans-serif;
                        padding: <?php echo esc($this->settings['paddingTop']).' '.esc($this->settings['paddingRight']).' '.esc($this->settings['paddingBottom']).' '.esc($this->settings['paddingLeft']); ?>;
                        margin: <?php echo esc($this->settings['marginTop']).' '.esc($this->settings['marginRight']).' '.esc($this->settings['marginBottom']).' '.esc($this->settings['marginLeft']); ?>;"
            >
                <p><?php echo nl2br(esc($this->settings['content'])); ?></p>
            </div>
        </div>
        
        <?php
        return ob_get_clean();
    }

    /**
     * Render how this module appears in the sidebar.
     */
    public function renderPreview() {
        $shortContent = (strlen($this->settings['content']) > 50)
            ? substr($this->settings['content'], 0, 50) . '...'
            : $this->settings['content'];
        return '<button class="module-selector-btn" data-module="PageTextModule">Text Module: '.esc($shortContent).'</button>';
    }

    /**
     * Render the settings panel (TinyMCE + alignment + width slider).
     */
    public function renderSettings() {
        ob_start(); ?>
        
        <!-- Text Content (TinyMCE) -->
        <div class="form-group">
            <label for="textContent">Text Content:</label>
            <textarea id="textContent" data-update="content"><?php echo esc($this->settings['content']); ?></textarea>
        </div>
        
        <!-- Text Box Alignment -->
        <div class="form-group">
            <label for="textBoxAlign">Text Box Position:</label>
            <select id="textBoxAlign" data-update="textBoxAlign">
                <option value="flex-start" <?php echo $this->settings['textBoxAlign'] === 'flex-start' ? 'selected' : ''; ?>>Left</option>
                <option value="center" <?php echo $this->settings['textBoxAlign'] === 'center' ? 'selected' : ''; ?>>Center</option>
                <option value="flex-end" <?php echo $this->settings['textBoxAlign'] === 'flex-end' ? 'selected' : ''; ?>>Right</option>
            </select>
        </div>
        
        <!-- Text Alignment -->
        <div class="form-group">
            <label for="textAlign">Text Alignment:</label>
            <select id="textAlign" data-update="textAlign">
                <option value="left" <?php echo $this->settings['textAlign'] === 'left' ? 'selected' : ''; ?>>Left</option>
                <option value="center" <?php echo $this->settings['textAlign'] === 'center' ? 'selected' : ''; ?>>Center</option>
                <option value="right" <?php echo $this->settings['textAlign'] === 'right' ? 'selected' : ''; ?>>Right</option>
                <option value="justify" <?php echo $this->settings['textAlign'] === 'justify' ? 'selected' : ''; ?>>Justify</option>
            </select>
        </div>
        
        <!-- Width Slider -->
        <div class="form-group">
            <label for="textWidth">Text Width (%):</label>
            <input type="range" id="textWidth" min="10" max="100" value="<?php echo intval($this->settings['width']); ?>" data-update="width">
            <span id="textWidthValue"><?php echo intval($this->settings['width']); ?>%</span>
        </div>
        
        <!-- Padding Controls -->
        <div class="form-group">
            <label>Padding:</label>
            <div class="padding-controls">
                <input type="text" id="paddingTop" data-update="paddingTop" value="<?php echo esc($this->settings['paddingTop']); ?>" placeholder="Top">
                <input type="text" id="paddingRight" data-update="paddingRight" value="<?php echo esc($this->settings['paddingRight']); ?>" placeholder="Right">
                <input type="text" id="paddingBottom" data-update="paddingBottom" value="<?php echo esc($this->settings['paddingBottom']); ?>" placeholder="Bottom">
                <input type="text" id="paddingLeft" data-update="paddingLeft" value="<?php echo esc($this->settings['paddingLeft']); ?>" placeholder="Left">
            </div>
        </div>
        
        <!-- Margin Controls -->
        <div class="form-group">
            <label>Margin:</label>
            <div class="margin-controls">
                <input type="text" id="marginTop" data-update="marginTop" value="<?php echo esc($this->settings['marginTop']); ?>" placeholder="Top">
                <input type="text" id="marginRight" data-update="marginRight" value="<?php echo esc($this->settings['marginRight']); ?>" placeholder="Right">
                <input type="text" id="marginBottom" data-update="marginBottom" value="<?php echo esc($this->settings['marginBottom']); ?>" placeholder="Bottom">
                <input type="text" id="marginLeft" data-update="marginLeft" value="<?php echo esc($this->settings['marginLeft']); ?>" placeholder="Left">
            </div>
        </div>
        
        <!-- TinyMCE Initialization + Data-Update Logic -->
        <script>
        // Initialize TinyMCE
        tinymce.init({
            selector: '#textContent',
            menubar: false,
            toolbar: 'bold italic underline | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent',
            setup: function (editor) {
                // On text input, update the preview
                editor.on('input', function () {
                    const selectedP = document.querySelector('.selected .text-container p');
                    if (selectedP) {
                        selectedP.innerHTML = editor.getContent();
                    }
                });
            }
        });

        // Listen to all data-update inputs
        document.querySelectorAll('[data-update]').forEach(el => {
            el.addEventListener('input', function () {
                const setting = this.getAttribute('data-update');
                const container = document.querySelector('.selected .text-container');
                const outerModule = document.querySelector('.selected');

                if (!container || !outerModule) return;

                // If we're editing the main text content via TinyMCE, skip
                if (setting === 'content') {
                    // Already handled by TinyMCE event
                    return;
                }

                // If changing text box alignment (justify-content)
                if (setting === 'textBoxAlign') {
                    outerModule.style.justifyContent = this.value;
                    return;
                }

                // If changing the 'width', add a '%' to the value
                if (setting === 'width') {
                    container.style.width = this.value + '%';
                    document.getElementById('textWidthValue').textContent = this.value + '%';
                    return;
                }

                // If changing padding or margin parts
                if (setting.startsWith('padding') || setting.startsWith('margin')) {
                    // We'll reconstruct the entire style attribute for padding or margin
                    let top    = document.getElementById('paddingTop')?.value || '10px';
                    let right  = document.getElementById('paddingRight')?.value || '10px';
                    let bottom = document.getElementById('paddingBottom')?.value || '10px';
                    let left   = document.getElementById('paddingLeft')?.value || '10px';

                    if (setting.startsWith('margin')) {
                        top    = document.getElementById('marginTop')?.value || '10px';
                        right  = document.getElementById('marginRight')?.value || 'auto';
                        bottom = document.getElementById('marginBottom')?.value || '10px';
                        left   = document.getElementById('marginLeft')?.value || 'auto';

                        container.style.margin = top + ' ' + right + ' ' + bottom + ' ' + left;
                        return;
                    }

                    // otherwise it's padding
                    container.style.padding = top + ' ' + right + ' ' + bottom + ' ' + left;
                    return;
                }

                // For everything else (fontSize, fontWeight, textAlign, etc.):
                container.style[setting] = this.value;
            });
        });
        </script>
        <?php
        return ob_get_clean();
    }
}
